use rand::distr::{Alphanumeric, SampleString};
use sqlx::{Postgres, QueryBuilder};
use validator::Validate;

#[derive(Debug, Validate)]
pub struct Member {
    #[validate(length(equal = 7))]
    pub id: String,
    pub first_name: String,
    pub full_name: String,
    pub registration_token: Option<String>,
    pub diploma: Option<String>,
    pub hours: Vec<String>,
    pub groups: Vec<String>,
}

impl Member {
    pub async fn insert_multiple(
        transaction: &mut sqlx::Transaction<'_, Postgres>,
        members: Vec<Self>,
    ) -> Result<(), sqlx::Error> {
        let mut query_builder = QueryBuilder::new(
            "INSERT INTO members(id, first_name, full_name, registration_token, diploma, hours, groups) "
        );

        query_builder.push_values(members.into_iter(), |mut b, member| {
            let registration_token = Alphanumeric.sample_string(&mut rand::rng(), 16);

            b.push_bind(member.id);
            b.push_bind(member.first_name);
            b.push_bind(member.full_name);
            b.push_bind(registration_token);
            b.push_bind(member.diploma);
            b.push_bind(member.hours);
            b.push_bind(member.groups);
        });

        let query = query_builder.build();
        query.execute(&mut **transaction).await?;

        Ok(())
    }

    pub async fn update_multiple(
        transaction: &mut sqlx::Transaction<'_, Postgres>,
        members: Vec<Self>,
    ) -> Result<(), sqlx::Error> {
        for member in members {
            sqlx::query!("UPDATE ONLY members SET first_name = $1, full_name = $2, diploma = $3, hours = $4, groups = $5 WHERE id = $6", member.first_name, member.full_name, member.diploma, &member.hours, &member.groups, member.id).execute(&mut **transaction).await?;
        }

        Ok(())
    }
}
